!
!   EAM FORCE & ENERGY CALCULATION
!   for Iron
!   Finnis & Sinclair potential (Phil. Mag. A 50 (1984) p.45)
!   calculation of force, total energy, d(mat)
!   New bookkeep algorithm
!   
!   Feb. 24 (2006)
!   Stomic stress calculation is implemented
!   Oct. 5, 2010
!   A bug in stress calculation for a small cell is fixed

      module fs_function
      double precision, allocatable :: pairx(:),pairy(:),pairc(:,:),pairxn(:)
      double precision, allocatable :: denx(:),deny(:),denc(:,:),denxn(:)
      double precision, allocatable :: embedx(:),embedy(:),embedc(:,:),embedxn(:)
      double precision, allocatable :: spd2(:)
      integer, parameter :: mesh=1000000
      double precision :: pair(mesh),paird(mesh)
      double precision :: den(mesh),dend(mesh)
      double precision :: embed(mesh),embedd(mesh)
      dimension DF(2), IOPT(2), EC(4)
      double precision :: pairmin,pairmax
      double precision :: denmin,denmax
      double precision :: embedmin,embedmax
      logical, save :: fs_setup_done = .false.
      end module

      subroutine eam_FS
      use fs_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: rhob(n) ! rho-bar_i (density)
      double precision :: ki,kif
      integer :: istat

      istat=1

!  Functions setup
      if (.not.fs_setup_done) then
         print *,'FS setup'
         call fs_setup
      endif

      if (rc<0.0) then
         stop 'Fatal error: please specify rc in SETDAT'
      endif
      rc2=rc**2

      rhob=0.0d0

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   First Loop: Chg dns calc
      kns=0
      idbg=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  ii=int(dble(mesh)*(r/ang-denmin)/(denmax-denmin))
                  rhob(k)=rhob(k)+den(ii)
                  if (j/=k) then
                     rhob(j)=rhob(j)+den(ii)
                  endif
               endif
            endif
 4300    continue
 4200 continue

!     Calculation of potential energy
      do i=1,n
         ii=int(dble(mesh)*(rhob(i)-embedmin)/(embedmax-embedmin))
         enpoteach(i)=embed(ii)*ev
      enddo

!   Second Loop: Force calc
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  ii=int(dble(mesh)*(r/ang-pairmin)/(pairmax-pairmin))
                  vij=pair(ii)*ev
                  dv=paird(ii)
                  enpoteach(i)=enpoteach(i)+vij/2.0d0
                  vijsum=vijsum+vij/2.0d0
                  if (j/=i) then
                     enpoteach(j)=enpoteach(j)+vij/2.0d0
                     vijsum=vijsum+vij/2.0d0
                  endif

                  ii=int(dble(mesh)*(rhob(i)-embedmin)/(embedmax-embedmin))
                  jj=int(dble(mesh)*(rhob(j)-embedmin)/(embedmax-embedmin))
                  df_i=embedd(ii)
                  df_j=embedd(jj)

                  ii=int(dble(mesh)*(r/ang-denmin)/(denmax-denmin))
                  dr=dend(ii)

                  f=-(df_i*dr+df_j*dr+dv)*ev/ang

                  ki=f/r
                  if (j==i) ki=ki/2.0d0
                  kif=ki/2.0d0
                  fx(i)=fx(i)+ki*xx
                  fy(i)=fy(i)+ki*yy
                  fz(i)=fz(i)+ki*zz
                  fx(j)=fx(j)-ki*xx
                  fy(j)=fy(j)-ki*yy
                  fz(j)=fz(j)-ki*zz
                  satom(i,1,1)=satom(i,1,1)-kif*xx*xx
                  satom(i,1,2)=satom(i,1,2)-kif*yy*xx
                  satom(i,2,2)=satom(i,2,2)-kif*yy*yy
                  satom(i,1,3)=satom(i,1,3)-kif*zz*xx
                  satom(i,2,3)=satom(i,2,3)-kif*zz*yy
                  satom(i,3,3)=satom(i,3,3)-kif*zz*zz
                  satom(j,1,1)=satom(j,1,1)-kif*xx*xx
                  satom(j,1,2)=satom(j,1,2)-kif*yy*xx
                  satom(j,2,2)=satom(j,2,2)-kif*yy*yy
                  satom(j,1,3)=satom(j,1,3)-kif*zz*xx
                  satom(j,2,3)=satom(j,2,3)-kif*zz*yy
                  satom(j,3,3)=satom(j,3,3)-kif*zz*zz
               endif
            endif
 4600    continue
 4500 continue

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo
!      write (10,*) 'EMT force calculation end.'
      end



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
!     FS SETUP
!     READ FUNCTIONS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
      subroutine fs_setup
      use fs_function
      use constants
      implicit double precision(a-h,o-z)
      character (1) :: ichar
      double precision :: fsaa,fsbet,fsd,fsc,fsc0,fsc1,fsc2
      
      if ((asp(1)=='fe').or.(asp(1)=='Fe')) then
         fsaa = 1.828905d0      ! eV
         fsd  = 3.569745d0      ! angstrom
         fsbet= 1.8d0           ! dimensionless
         fsc  = 3.4d0           ! angstrom
         fsc0 = 1.2371147d0     ! dimensionless
         fsc1 =-0.3592185d0     ! dimensionless
         fsc2 =-0.0385607d0     ! dimensionless
      else 
         print *,'eam_FS is prepared only for Fe'
         print *, asp(1),' is not supported'
         stop
      endif
! pair function
      mesh_pair=2000
      allocate(pairx(mesh_pair),pairy(mesh_pair),pairc(mesh_pair,3))
      rmin=1.0d0; rmax=4.0d0    ! range of r to be prepared as mesh data
      rinc=(rmax-rmin)/(mesh_pair-1)
      pairmin=rmin; pairmax=rmax
      r=rmin
      do i=1,mesh_pair
         pairx(i)=r
         if (r<=fsc) then
            pairy(i)=(r-fsc)**2*(fsc0+fsc1*r+fsc2*r**2)
         else
            pairy(i)=0.0d0
         endif
         r=r+rinc
      enddo
! finer mesh set
      allocate(pairxn(mesh),spd2(mesh))
      x=pairmin
      d=(pairmax-pairmin)/dble(mesh-1)
      do i=1,mesh
         pairxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3
      iopt(2)=3
      call SPLC(pairx, mesh_pair, pairy, DF, IOPT, pairc, mesh_pair, IER)
      call SPLF(pairx,mesh_pair,pairy,pairc,mesh_pair,pairxn,mesh,pair, IER)
      call SPLD(pairx,mesh_pair,pairc,mesh_pair,pairxn,mesh,paird,spd2, IER)
      deallocate (pairxn,spd2)
      deallocate(pairx,pairy,pairc)

! Cutoff radius set
      rc=max(fsc*ang,fsd*ang)
      rc2=rc**2

! density function
      mesh_den=2000
      allocate(denx(mesh_den),deny(mesh_den),denc(mesh_den,3))
      rmin=1.0d0; rmax=4.0d0    ! range of r to be prepared as mesh data
      rinc=(rmax-rmin)/(mesh_den-1)
      denmin=rmin; denmax=rmax
      r=rmin
      do i=1,mesh_den
         denx(i)=r
         if (r<=fsd) then
            deny(i)=(r-fsd)**2+fsbet*(r-fsd)**3/fsd
         else
            deny(i)=0.0d0
         endif
         r=r+rinc
      enddo
! finer mesh set
      allocate(denxn(mesh),spd2(mesh))
      x=denmin
      d=(denmax-denmin)/dble(mesh-1)
      do i=1,mesh
         denxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3
      iopt(2)=3
      call SPLC(denx, mesh_den, deny, DF, IOPT, denc, mesh_den, IER)
      call SPLF(denx,mesh_den,deny,denc,mesh_den,denxn,mesh,den, IER)
      call SPLD(denx,mesh_den,denc,mesh_den,denxn,mesh,dend,spd2, IER)
      deallocate (denxn,spd2)
      deallocate(denx,deny,denc)

! Embed function
      mesh_embed=2000
      allocate(embedx(mesh_embed),embedy(mesh_embed),embedc(mesh_embed,3))
      rhomin=0.0d0; rhomax=20.0d0 ! range of rho to be prepared as mesh data
      rhoinc=(rhomax-rhomin)/(mesh_embed-1)
      embedmin=rhomin; embedmax=rhomax
      rho=rhomin
      do i=1,mesh_embed
         embedx(i)=rho
         embedy(i)=-fsaa*dsqrt(rho)
         rho=rho+rhoinc
      enddo
! finer mesh set
      allocate(embedxn(mesh),spd2(mesh))
      x=embedmin
      d=(embedmax-embedmin)/dble(mesh-1)
      do i=1,mesh
         embedxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3
      iopt(2)=3
      call SPLC(embedx, mesh_embed, embedy, DF, IOPT, embedc, mesh_embed, IER)
      call SPLF(embedx,mesh_embed,embedy,embedc,mesh_embed,embedxn,mesh,embed, IER)
      call SPLD(embedx,mesh_embed,embedc,mesh_embed,embedxn,mesh,embedd,spd2, IER)
      deallocate (embedxn,spd2)
      deallocate(embedx,embedy,embedc)

      fs_setup_done=.true.

      end

      
