!---calculation of force, total energy, and d(mat)---
!      subroutine morse(nmax,nbook,n,rx,ry,rz,fx,fy,fz,idpt,knum
!     &     ,hmat,rc,rc2,dmat,px,py,pz
!     &     ,ep,al,ro
!     &     ,enpoteach,enpot,wm,stressx,stressy,stressz,ef
!     &     ,iban)
!   Oct. 5, 2010
!   A bug in stress calculation for a small cell is fixed
module morse_param
  double precision :: ep, al, ro
end module morse_param

subroutine morse
  use morse_param
  use variables
  use constants
  use bookkeep_param
  implicit double precision (a-h,o-z)
  double precision :: val1, val2, val3
  double precision :: ki,kif
  
  if ((potential_arg(1:1)=='X').or.(potential_arg(1:1)=='x')) then
     ep = param(1)
     al = param(2)
     ro = param(3)
  else
! Parameters taken from Phys. Rev. 114 (1959) 687 by Girfalco and Weizer
     if (asp(1)=='Al') then
        ep = 0.2703d0 ! [eV]
        al = 1.1646d0 ! [ang^-1]
        ro = 3.253d0  ! [ang]
     else if (asp(1)=='Cu') then
        ep = 0.3429d0; al = 1.3588d0; ro = 2.866d0
     else if (asp(1)=='Fe') then
        ep = 0.4174d0; al = 1.3885d0; ro = 2.845d0
     else if (asp(1)=='Ni') then
        ep = 0.4205d0; al = 1.4199d0; ro = 2.780d0
     else if (asp(1)=='W ') then
        ep = 0.9906d0; al = 1.4116d0; ro = 3.032d0
     else if (asp(1)=='Cr') then
        ep = 0.4414d0; al = 1.5721d0; ro = 2.754d0
     else
        print *,'Error: ',asp(1),' is not implemented in morse.F'
        stop
     endif
  endif
  rc = ro*3.6d0*ang

  if (potparam1/=0.0d0) ep = potparam1
  if (potparam2/=0.0d0) al = potparam2
  if (potparam3/=0.0d0) ro = potparam3
  if (potparam4/=0.0d0) rc = potparam4*ang

  if (rc<0.0) then
     stop 'Fatal error: please specify rc in SETDAT'
  endif
  rc2=rc**2

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   Loop: Energy and force calc
      enpoteach(1:n)=0.0d0
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0

      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
!                  ii=int(dble(mesh)*(r/ang-pairmin)/(pairmax-pairmin))
!                  vij=pair(ii)*ev
!                  dv=paird(ii)

                  rang=r/ang
                  vij=ep*(exp(-2.0d0*al*(rang-ro))-2.0d0*exp(-al*(rang-ro)))*ev
                  dv=-2.0d0*al*ep*(exp(-2.0d0*al*(rang-ro))-exp(-al*(rang-ro)))

                  enpoteach(i)=enpoteach(i)+vij/2.0d0
                  if (j/=i) then
                     enpoteach(j)=enpoteach(j)+vij/2.0d0
                  endif

!               f=2.0d0*al*ep
!     &              *(exp(-2.0d0*al*(r-ro))-exp(-al*(r-ro)))
!               ene=ep*(exp(-2.0d0*al*(r-ro))
!     &              -2.0d0*exp(-al*(r-ro)))
!               enpoteach(i)=enpoteach(i)+ene
!               enpoteach(j)=enpoteach(j)+ene

                  f=-(dv)*ev/ang

                  ki=f/r
                  if (j==i) ki=ki/2.0d0
                  kif=ki/2.0d0
                  fx(i)=fx(i)+ki*xx
                  fy(i)=fy(i)+ki*yy
                  fz(i)=fz(i)+ki*zz
                  fx(j)=fx(j)-ki*xx
                  fy(j)=fy(j)-ki*yy
                  fz(j)=fz(j)-ki*zz
                  satom(i,1,1)=satom(i,1,1)-kif*xx*xx
                  satom(i,1,2)=satom(i,1,2)-kif*yy*xx
                  satom(i,2,2)=satom(i,2,2)-kif*yy*yy
                  satom(i,1,3)=satom(i,1,3)-kif*zz*xx
                  satom(i,2,3)=satom(i,2,3)-kif*zz*yy
                  satom(i,3,3)=satom(i,3,3)-kif*zz*zz
                  satom(j,1,1)=satom(j,1,1)-kif*xx*xx
                  satom(j,1,2)=satom(j,1,2)-kif*yy*xx
                  satom(j,2,2)=satom(j,2,2)-kif*yy*yy
                  satom(j,1,3)=satom(j,1,3)-kif*zz*xx
                  satom(j,2,3)=satom(j,2,3)-kif*zz*yy
                  satom(j,3,3)=satom(j,3,3)-kif*zz*zz
               endif
            endif
 4600    continue
 4500 continue

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

!      write (10,*) 'EMT force calculation end.'
      end



