! Calculation of Phonon
! mul{x,y,z} = number of replica cells in each direction
! qq{x,y,z}  = wave number
subroutine phonon(mulx,muly,mulz,qqx,qqy,qqz)
  use variables
  use constants
  use bookkeep_param
  implicit double precision (a-h,o-z)
  double precision, allocatable :: rx_store(:),ry_store(:),rz_store(:)
  character (2), allocatable :: asp_store(:)
  double precision, allocatable :: wm_store(:)
  double precision :: hmat_store(3,3)
  integer, allocatable :: n0rep(:)
  complex :: wav
  double precision :: qqx,qqy,qqz
  double precision, allocatable :: w(:,:), rwork(:), ph(:)
  double complex, allocatable :: vl(:,:), vr(:,:), work(:)
  integer :: info

  del = a/10.0d0

! Preparation of replica cells
  allocate (rx_store(n),ry_store(n),rz_store(n),asp_store(n),wm_store(n))
  rx_store=rx; ry_store=ry; rz_store=rz; asp_store=asp; wm_store=wm
  hmat_store=hmat
  deallocate (rx,ry,rz,fx,fy,fz,asp,qx,qy,qz,wm,enpoteach,satom)
  n0=n ! n0 is number of atoms in fundamental cell
  ncel=(mulx*2+1)*(muly*2+1)*(mulz*2+1) ! ncel is number of replica and fundamental cells
  n=n0*ncel ! n is number of atoms in fundamental and replica cells
  allocate (n0rep(ncel-1)) ! stores id of 0st atom in each replica cell
  allocate (rx(n),ry(n),rz(n),fx(n),fy(n),fz(n),asp(n),qx(n),qy(n),qz(n),wm(n))
  allocate (enpoteach(n),satom(n,3,3))

  rx(1:n0)=rx_store(1:n0)
  ry(1:n0)=ry_store(1:n0)
  rz(1:n0)=rz_store(1:n0)
  asp(1:n0)=asp_store(1:n0)
  wm(1:n0)=wm_store(1:n0)

  iii=n0
  idrep=0
  do iix=-mulx,mulx
     do iiy=-muly,muly
        do iiz=-mulz,mulz
           if ((iix/=0).or.(iiy/=0).or.(iiz/=0)) then
              idrep=idrep+1 ! id of replica cell
              n0rep(idrep)=n0*idrep
              do i=1,n0
                 iii=iii+1
                 rx(iii)=rx(i)+hmat(1,1)*iix+hmat(1,2)*iiy+hmat(1,3)*iiz
                 ry(iii)=ry(i)+hmat(2,1)*iix+hmat(2,2)*iiy+hmat(2,3)*iiz
                 rz(iii)=rz(i)+hmat(3,1)*iix+hmat(3,2)*iiy+hmat(3,3)*iiz
                 asp(iii)=asp(i)
                 wm(iii)=wm(i)
              enddo
           endif
        enddo
     enddo
  enddo
  do i=1,3
     hmat(i,1)=hmat_store(i,1)*(mulx*2+1)
     hmat(i,2)=hmat_store(i,2)*(muly*2+1)
     hmat(i,3)=hmat_store(i,3)*(mulz*2+1)
  enddo
  call inverse(hmat,hinmat)
  call calcvolume
  call mkq

  call bookkeep

! Calculation of force constants
  if (allocated(fconst)) go to 1000
  allocate (fconst(n0,3,n0,3,ncel))
  deallocate (rx_store,ry_store,rz_store)
  allocate (rx_store(n),ry_store(n),rz_store(n))
  call potential

  rx_store=rx; ry_store=ry; rz_store=rz ! Atom position save
  fconst=0.0d0

  do i=1,n0
     print *,'Fconst calc',i
     do j=1,n0
        idcel=0
        idrep=0
        print *,'hoge',i,j
        do iix=-mulx,mulx
           do iiy=-muly,muly
              do iiz=-mulz,mulz
                 idcel=idcel+1
                 if ((iix==0).and.(iiy==0).and.(iiz==0)) then
                    jl=j
                 else
                    idrep=idrep+1
                    jl=j+n0rep(idrep)
                 endif
                 xxx=rx_store(jl)-rx_store(i)
                 yyy=ry_store(jl)-ry_store(i)
                 zzz=rz_store(jl)-rz_store(i)
                 dist2=xxx**2+yyy**2+zzz**2; rc2=rc**2

                 if (dist2 < rc2) then
                 
                    rx=rx_store; ry=ry_store; rz=rz_store
                    rx(jl)=rx_store(jl)+del
                    call potential
                    fconst(i,1,j,1,idcel) = -fx(i)/del
                    fconst(i,2,j,1,idcel) = -fy(i)/del
                    fconst(i,3,j,1,idcel) = -fz(i)/del
                    
                    rx=rx_store; ry=ry_store; rz=rz_store
                    ry(jl)=ry_store(jl)+del
                    call potential
                    fconst(i,1,j,2,idcel) = -fx(i)/del
                    fconst(i,2,j,2,idcel) = -fy(i)/del
                    fconst(i,3,j,2,idcel) = -fz(i)/del
                    
                    rx=rx_store; ry=ry_store; rz=rz_store
                    rz(jl)=rz_store(jl)+del
                    call potential
                    fconst(i,1,j,3,idcel) = -fx(i)/del
                    fconst(i,2,j,3,idcel) = -fy(i)/del
                    fconst(i,3,j,3,idcel) = -fz(i)/del
                 
                 endif

              enddo
           enddo
        enddo
     enddo
  enddo
  rx=rx_store; ry=ry_store; rz=rz_store
  
! Calculation of dynamical matrix
1000 continue
  if (.not.allocated(dynmat)) allocate(dynmat(3*n0,3*n0))
  allocate (w(1,n0*3),vl(1,n0*3),vr(1,n0*3),work(n0*3*2),rwork(n0*3*2))
  allocate (ph(n0*3))
  open (30,FILE='phonon.d')

  do iii=1,30

     qqx=2.0d0*pi/a*dble(iii)/dble(10.0)

  dynmat = 0.0d0
  do i=1,n0
     do j=1,n0
        ia=3*(i-1); ib=3*(j-1)
        idcel=0
        idrep=0
        do iix=-mulx,mulx
           do iiy=-muly,muly
              do iiz=-mulz,mulz
                 idcel=idcel+1
                 if ((iix==0).and.(iiy==0).and.(iiz==0)) then
                    jl=j
                 else
                    idrep=idrep+1
                    jl=j+n0rep(idrep)
                 endif
                 xxx=rx(jl)-rx(i); yyy=ry(jl)-ry(i); zzz=rz(jl)-rz(i)
                 qr=qqx*xxx+qqy*yyy+qqz*zzz
                 wav=exp(cmplx(0.0,1.0)*qr)
                 dynmat(ia+1,ib+1)=dynmat(ia+1,ib+1)+fconst(i,1,j,1,idcel)*wav
              enddo
           enddo
        enddo
     enddo
  enddo
  call ZGEEV('N','N',n0*3,dynmat,n0*3,w,vl,1,vr,1,work,n0*3*2,rwork,info)
  if (info/=0) stop 'error'
  ph(1:n0*3)=w(1,1:n0*3)
  call psort(n0*3,ph)
  
  write (30,200) real(qqx)*a,(real(ph(i)),i=1,10)
  write (30,200) real(qqx)*a,(real(ph(i)),i=11,20)
  write (30,200) real(qqx)*a,(real(ph(i)),i=21,24)
200 format (11(f10.5,1x))

  enddo

  stop

end subroutine phonon

subroutine psort(n,val)
  integer :: n ! Number of values
  double precision :: val(n) ! Array of values

  do i=1,n-1
     do j=i+1,n
        if (val(i)>val(j)) then
           tmp=val(i)
           val(i)=val(j)
           val(j)=tmp
        endif
     enddo
  enddo

end subroutine psort
