!
! SUBROUTINE WRITEDATA
!
! This subroutine controls all outputs to both files and the console
!  together with opening the files at the beginning of the program run.
! This enables the users to easily understand
!  which data are stored to which files.
! Try not to insert commands for printing or writing in main.F.
!
      subroutine writedata
      use variables
      use constants
      use bookkeep_param
      double precision, save :: xx0,yy0 !used for memorize initial energy
      double precision :: xx,yy,zz
      character (3) :: stri
      
! Commands conducted at the beginning of the program run.
! Opening files and writing the settings.
      if (step == step_start) then
         open (21,FILE='energy.d')
         open (22,FILE='temp.d')
         open (25,FILE='cell.d')
         open (26,FILE='stress.d')
         if (diffusion) then
            open (50,FILE='msd.d')
         endif

         if (continue_md) then
            do while(.true.)
               read (21,*,end=111) xx
            enddo
111         continue
            do while(.true.)
               read (22,*,end=222) xx
            enddo
222         continue
            do while(.true.)
               read (25,*,end=333) xx
            enddo
333         continue
            do while(.true.)
               read (26,*,end=444) xx
            enddo
444         continue
            if (diffusion) then
               do while(.true.)
                  read (50,*,end=501) xx
               enddo
501            continue
            endif
            
         else

         write (6,*) 'Number of atoms',n
!         write (6,*) 'Cut off radius',rc/ang
!         write (6,*) 'Cut off radius for bookkeep',frc/ang
         write (6,*) 'Initialize done. Loop start.'

         open (30,FILE='conf-init.d')
         if (multisystem==0) then
            write (30,*) asp(1)
         else
            write (30,*) 'Multi'
         endif
         write (30,*) potential_func, potential_arg
         write (30,*) n
         do j=1,3
            xx=hmat(1,j)/ang; yy=hmat(2,j)/ang; zz=hmat(3,j)/ang
            write (30,*) xx,yy,zz
         enddo
         do i=1,n
            xx=rx(i)/ang; yy=ry(i)/ang; zz=rz(i)/ang
            write (30,*) xx,yy,zz
         enddo
         close (30)
         call write_xsf(30,'conf-init.xsf')
         call write_cfgs(30,'conf-init.cfg')
         call write_xsf(30,'conf.xsf')
         call write_cfgs(30,'conf.cfg')

         endif

         open (30,FILE='OUT')
         write (30,'(10("-"),"MD SETTING",10("-"))') 
         if (multisystem==0) then
            write (30,'("Atom species ",a2)') asp(1)
         else
            write (30,'("Multisystem")')
         endif
         write (30,'("Potential function ",a7)') potential_func
         write (30,'("Potential option ",a7)') potential_arg
         write (30,'("Number of atoms ",i10)') n
         write (30,'("Periodic boundary (xyz): ",L1,1x,L1,1x,L1)') pbcx,pbcy,pbcz
         write (30,'(30("-"))') 
         write (30,'("Time step ",f10.3," fs")') dt/1.0d-15
         write (30,'("Interval of bookkeeping ",i8)') nbk
         write (30,'("Number of iterations ",i10)') step_ite
         if (continue_md) then
            write (30,'("Continue MD using CONT")')
         else
            write (30,'("Start MD from CONFIG")')
            if (ini_velocity) then
               write (30,'("Initial velocity is given")')
            else
               write (30,'("Initial velocity is NOT given")')
            endif
         endif
         write (30,'(30("-"))') 
         if (algorithm==0) then
            write (30,'("Velocity Verlet, NVE")')
         else if (algorithm==1) then
            write (30,'("Gear P-C, NVE")')
         else if (algorithm==2) then
            write (30,'("Gear, NVT (Nose thermostat)")')
         else if (algorithm==3) then
            write (30,'("Gear, NPH (Andersen pressure control)")')
         else if (algorithm==4) then
            write (30,'("Gear, NPT (Nose and Andersen)")')
         else if (algorithm==5) then
            write (30,'("Gear, NPH-PR (Parrinello-Rahman)")')
         else if (algorithm==6) then
            write (30,'("Gear, NPT-PR (Nose and Parrinello-Rahman)")')
         else if (algorithm==7) then
            write (30,'("Stress relaxation by annealing, no atom motion")')
         else if (algorithm==8) then
            write (30,'("Stress relaxation by annealing, atom motion by verlet")')
         else if (algorithm==107) then
            write (30,'("Gear, Stress relaxation by NPH-PR & breaking")')
         endif
         if (vscale==0) then
            write (30,'("No velocity scaling ")')
         else
            write (30,'("Velocity scaling interval ",i10)') vscale
         endif
         if (damper==0) then
            write (30,'("No damper to atom motion ")')
         else
            write (30,'("Interval of damper to atom motion ",i10)') damper
         endif
         write (30,'("Cell constraint (xyz): ",L1,1x,L1,1x,L1)') cellconstx,cellconsty,cellconstz
         write (30,'(30("-"))') 
         if ((algorithm==2).or.(algorithm==4).or.(algorithm==6).or.(vscale==1)) then
            write (30,'("Target temperature ",f10.5," K")') temp_set
            write (30,'("Weight factor for temperature control ",f10.5)') temp_weight
         endif
         if ((algorithm==3).or.(algorithm==4)) then
            write (30,'("Target stress setting (hydrostatic) ",f10.5," MPa")') sgm_set/mpa
            write (30,'("Weight factor for stress control ",f10.5)') stress_weight
         else if (algorithm>=5) then
            write (30,'("Target stress setting (xx) ",f10.5," MPa")') sxx/mpa
            write (30,'("Target stress setting (yy) ",f10.5," MPa")') syy/mpa
            write (30,'("Target stress setting (zz) ",f10.5," MPa")') szz/mpa
            write (30,'("Target stress setting (yz) ",f10.5," MPa")') syz/mpa
            write (30,'("Target stress setting (zx) ",f10.5," MPa")') szx/mpa
            write (30,'("Target stress setting (xy) ",f10.5," MPa")') sxy/mpa
            write (30,'("Weight factor for stress control ",f10.5)') stress_weight
         endif
         write (30,'(30("-"))') 
         close (30)

         if (step>=step_end) then ! in case of "step_ite = 0"
            open (30,FILE='CONFIG.END')
            if (multisystem==0) then
               write (30,*) asp(1)
            else
               write (30,*) 'Multi'
            endif
            write (30,*) potential_func, potential_arg
            write (30,*) n
            write (30,*) scl_lat
            do j=1,3
               xx=hmat(1,j)/ang; yy=hmat(2,j)/ang; zz=hmat(3,j)/ang
               xx=xx/scl_lat; yy=yy/scl_lat; zz=zz/scl_lat
               write (30,900) xx,yy,zz
            enddo
            write (30,*) 'abs'
            do i=1,n
               xx=rx(i)/ang; yy=ry(i)/ang; zz=rz(i)/ang
               write (30,910) xx,yy,zz,asp(i),mfx(i),mfy(i),mfz(i)
            enddo
            close (30)
            open (30,FILE='CONFIG.END.FRA')
            call mkq
            if (multisystem==0) then
               write (30,*) asp(1)
            else
               write (30,*) 'Multi'
            endif
            write (30,*) potential_func, potential_arg
            write (30,*) n
            write (30,*) scl_lat
            do j=1,3
               xx=hmat(1,j)/ang; yy=hmat(2,j)/ang; zz=hmat(3,j)/ang
               xx=xx/scl_lat; yy=yy/scl_lat; zz=zz/scl_lat
               write (30,900) xx,yy,zz
            enddo
            write (30,*) 'fra'
            do i=1,n
               xx=qx(i); yy=qy(i); zz=qz(i)
               write (30,910) xx,yy,zz,asp(i),mfx(i),mfy(i),mfz(i)
            enddo
            close (30)
         endif
         return ! writing at initial step end
      endif

! Configuration output
      if (mod(step,10) == 1) then
         open (30,FILE='conf.d')
         if (multisystem==0) then
            write (30,*) asp(1)
         else
            write (30,*) 'Multi'
         endif
         write (30,*) potential_func, potential_arg
         write (30,*) n
         do j=1,3
            xx=hmat(1,j)/ang; yy=hmat(2,j)/ang; zz=hmat(3,j)/ang
            write (30,*) xx,yy,zz
         enddo
         do i=1,n
            xx=rx(i)/ang; yy=ry(i)/ang; zz=rz(i)/ang
            write (30,*) xx,yy,zz
         enddo
         close (30)
         open (20,FILE='conf.xyz')
         write (20,'(1x,i8)') n
         if (multisystem==0) then
            write (20,*) asp(1)
         else
            write (20,*) 'Multi'
         endif
         do i=1,n
            write (20,'(1x,a2,3f10.5)') asp(i),rx(i)/ang,ry(i)/ang,rz(i)/ang
         enddo
         close (20)
         call write_xsf(20,'conf.xsf')
         call write_cfgs(20,'conf.cfg')
         call write_cfge(20,'conf.cfge')
      endif
! Displacement output (for diffusion calculation)
      if ((diffusion).and.(mod(step,difout_int)==0)) then
         iii=(step-step_start)/difout_int; call conv_num_char2(iii,stri)
         open (30,FILE='displacement'//stri//'.d')
         do i=1,n
!            xx=(rx(i)+rxd(i))/ang; yy=(ry(i)+ryd(i))/ang; zz=(rz(i)+rzd(i))/ang
            xx=rxd(i)/ang; yy=ryd(i)/ang; zz=rzd(i)/ang
            write (30,*) xx,yy,zz
         enddo
         close (30)
      endif

! Energy output
      if (step==1) then
         xx0=enpot; yy0=enkin
      endif
      if ((mod(step,1) == 0).and.(loopflg==0)) then
!         write (6,'(i8,x,f15.8," eV",x,e10.4," A**3",x,f10.4," K",3f10.4)') step,enpot/ev/n,v/ang**3,temp
         write (6,'(i8,1x,f15.8," eV",1x,4f15.5," MPa",1x,f10.4," K",3f10.4)') &
              step,enpot/ev/n,sgmmat(1,1)/mpa,sgmmat(2,2)/mpa,sgmmat(3,3)/mpa,sgmmat(3,1)/mpa,temp
!         write (6,'(i8,x,f10.2," GPa",x,f10.2," GPa",x,f10.2," GPa")') step,sgmmat(1,1)/gpa,sgmmat(2,2)/gpa,sgmmat(3,3)/gpa
!         write (6,'(4(e18.8,x))') strmat_set(1,1),enpot,v,sgmmat(1,1)
!         write (11,'(i8,4(e18.8,x))') step,enpot-xx0,enkin-yy0,enpot+enkin-xx0-yy0
         write (21,'(i8,1x,e23.15,1x,e18.10,1x,2(e20.12,1x))') step,enpot/ev/n,enkin/ev/n,(enpot+enkin)/ev/n
         write (22,'(i8,1x,1e20.10)') step,temp
         write (25,'(i8,1x,9(f10.5,1x))') step,hmat(1,1)/ang,hmat(2,1)/ang,hmat(3,1)/ang &
              ,hmat(1,2)/ang,hmat(2,2)/ang,hmat(3,2)/ang,hmat(1,3)/ang,hmat(2,3)/ang,hmat(3,3)/ang
         write (26,'(i8,1x,9(f10.5,1x))') step,sgmmat(1,1)/gpa,sgmmat(2,2)/gpa,sgmmat(3,3)/gpa &
              ,sgmmat(1,2)/gpa,sgmmat(1,3)/gpa,sgmmat(2,1)/gpa,sgmmat(2,3)/gpa,sgmmat(3,1)/gpa,sgmmat(3,2)/gpa
         if (diffusion) then
            write (50,'(i10, 6e20.10)') step, msdx, msdy, msdz, msdxc, msdyc, msdzc
         endif

      endif

! All data dump for continuation
      if ((step>=step_end).or.(step_ite==0).or.(loopflg>0)) then
         open (30,FILE='LAST')
         if (multisystem==0) then
            write (30,*) asp(1)
         else
            write (30,*) 'Multi'
         endif
         write (30,*) potential_func, potential_arg
         write (30,*) step
         write (30,*) n
         do i=1,n
            write (30,910) rx(i),ry(i),rz(i),asp(i),mfx(i),mfy(i),mfz(i)
            write (30,900) vx(i),vy(i),vz(i)
            write (30,900) ax(i),ay(i),az(i)
            write (30,900) bx(i),by(i),bz(i)
            write (30,900) cx(i),cy(i),cz(i)
         enddo
         write (30,915) s,sv,sa,sb,sc
         write (30,915) v,vv,va,vb,vc
         do i=1,3; do j=1,3
            write (30,915) hmat(i,j),hvmat(i,j),hamat(i,j),hbmat(i,j),hcmat(i,j)
         enddo; enddo
         close (30)
         open (30,FILE='CONFIG.END')
         if (multisystem==0) then
            write (30,*) asp(1)
         else
            write (30,*) 'Multi'
         endif
         write (30,*) potential_func, potential_arg
         write (30,*) n
         write (30,*) scl_lat
         do j=1,3
            xx=hmat(1,j)/ang; yy=hmat(2,j)/ang; zz=hmat(3,j)/ang
            xx=xx/scl_lat; yy=yy/scl_lat; zz=zz/scl_lat
            write (30,900) xx,yy,zz
         enddo
         write (30,*) 'abs'
         do i=1,n
            xx=rx(i)/ang; yy=ry(i)/ang; zz=rz(i)/ang
            write (30,910) xx,yy,zz,asp(i),mfx(i),mfy(i),mfz(i)
         enddo
         close (30)
         open (30,FILE='CONFIG.END.FRA')
         call mkq
         if (multisystem==0) then
            write (30,*) asp(1)
         else
            write (30,*) 'Multi'
         endif
         write (30,*) potential_func, potential_arg
         write (30,*) n
         write (30,*) scl_lat
         do j=1,3
            xx=hmat(1,j)/ang; yy=hmat(2,j)/ang; zz=hmat(3,j)/ang
            xx=xx/scl_lat; yy=yy/scl_lat; zz=zz/scl_lat
            write (30,900) xx,yy,zz
         enddo
         write (30,*) 'fra'
         do i=1,n
            xx=qx(i); yy=qy(i); zz=qz(i)
            write (30,910) xx,yy,zz,asp(i),mfx(i),mfy(i),mfz(i)
         enddo
         close (30)
         open (30,FILE='LAST.VAR') ! parameters to be saved
         write (30,920) dt, fire_alph, ffdtmax, ifire
         close (30)
         if (diffusion) then
            open (30,FILE='LAST.DIF')
            do i=1,n
               write (30,900) rxd(i),ryd(i),rzd(i)
            enddo
            close (30)
         endif

      endif
! 900  format (3(e25.18,1x))
! 910  format (3(e25.18,1x),a3,1x,3(a2,1x))
! 915  format (5(e25.18))
! 920  format (3(e25.18,1x),i10)
 900  format (3(e26.19,1x))
 910  format (1x,3(e26.19,1x),a3,1x,3(a2,1x))
 915  format (5(e26.19,1x))
 920  format (3(e26.19,1x),i10)

      end

      subroutine write_xsf(ifile,filename)
      use variables
      use constants
      character(*) :: filename
      integer :: ifile,idim,atom_number
      double precision :: hart = 4.35982d-18 ! !Hr=4.35982d-18 J

! dimension
      idim=0
      if (pbcx) idim=idim+1
      if (pbcy) idim=idim+1
      if (pbcz) idim=idim+1

      open (ifile,FILE=filename)
      write (ifile,'(1x,a7)') 'CRYSTAL'
      write (ifile,'(1x,a7)') 'PRIMVEC'
      write (ifile,'(3f10.5)') hmat(1,1)/ang,hmat(2,1)/ang,hmat(3,1)/ang
      write (ifile,'(3f10.5)') hmat(1,2)/ang,hmat(2,2)/ang,hmat(3,2)/ang
      write (ifile,'(3f10.5)') hmat(1,3)/ang,hmat(2,3)/ang,hmat(3,3)/ang
      write (ifile,'(1x,a9)') 'PRIMCOORD'
      write (ifile,'(1x,i8,i8)') n,1
      do i=1,n
         write (ifile,'(i3,1x,6f10.5)') atom_number(asp(i)),rx(i)/ang,ry(i)/ang,rz(i)/ang &
              ,fx(i)/hart*ang,fy(i)/hart*ang,fz(i)/hart*ang
      enddo

      close (ifile)
      end


! Write standard cfg format file
      subroutine write_cfgs(ifile,filename)
      use variables
      use constants
      character(*) :: filename
      integer :: ifile

      open (ifile,FILE=filename)
      write (ifile,'(a22,i10)') 'Number of particles = ',n
      write (ifile,'(a4,f10.5,a9)') 'A = ',scl_lat,' Angstrom'
      write (ifile,'(a10,f10.5)') 'H0(1,1) = ',hmat(1,1)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(1,2) = ',hmat(2,1)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(1,3) = ',hmat(3,1)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(2,1) = ',hmat(1,2)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(2,2) = ',hmat(2,2)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(2,3) = ',hmat(3,2)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(3,1) = ',hmat(1,3)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(3,2) = ',hmat(2,3)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(3,3) = ',hmat(3,3)/ang/scl_lat

      call mkq
      do i=1,n
!         qvx=hinmat(1,1)*vx(i)+hinmat(1,2)*vy(i)+hinmat(1,3)*vz(i)
!         qvy=hinmat(2,1)*vx(i)+hinmat(2,2)*vy(i)+hinmat(2,3)*vz(i)
!         qvz=hinmat(3,1)*vx(i)+hinmat(3,2)*vy(i)+hinmat(3,3)*vz(i)
!         write (ifile,'(f10.3,x,a2,x,6f12.8)') wm(i)/am,asp,qx(i),qy(i),qz(i),qvx,qvy,qvz
         write (ifile,'(f10.3,1x,a2,1x,6f15.8)') wm(i)/am,asp(i),qx(i),qy(i),qz(i),0.,0.,0.
      enddo

      close (ifile)

      end

! Write extended cfg format file
      subroutine write_cfge(ifile,filename)
      use variables
      use constants
      character(*) :: filename
      integer :: ifile
      double precision :: shydro,smises

      open (ifile,FILE=filename)
      write (ifile,'(a22,i10)') 'Number of particles = ',n
      write (ifile,'(a4,f10.5,a9)') 'A = ',scl_lat,' Angstrom'
      write (ifile,'(a10,f10.5)') 'H0(1,1) = ',hmat(1,1)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(1,2) = ',hmat(2,1)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(1,3) = ',hmat(3,1)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(2,1) = ',hmat(1,2)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(2,2) = ',hmat(2,2)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(2,3) = ',hmat(3,2)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(3,1) = ',hmat(1,3)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(3,2) = ',hmat(2,3)/ang/scl_lat
      write (ifile,'(a10,f10.5)') 'H0(3,3) = ',hmat(3,3)/ang/scl_lat

      write (ifile,'(a13)')       '.NO_VELOCITY.'
      write (ifile,'(a15)')       'entry_count = 7'
      write (ifile,'(a24)')       'auxiliary[0] = kine [eV]'
      write (ifile,'(a24)')       'auxiliary[1] = pote [eV]'
      write (ifile,'(a26)')       'auxiliary[2] = hydro [MPa]'
      write (ifile,'(a26)')       'auxiliary[3] = mises [MPa]'

      call mkq
      do i=1,n
!         write (ifile,'(f10.3,x,a2,x,6f15.8)') wm(i)/am,asp(i),qx(i),qy(i),qz(i),0,0,0
         write (ifile,'(f10.3)') wm(i)/am
         write (ifile,'(a2)') asp(i)
         shydro = satom(i,1,1) + satom(i,2,2) + satom(i,3,3)
         smises = (satom(i,1,1)-satom(i,2,2))**2 + (satom(i,2,2)-satom(i,3,3))**2 &
              + (satom(i,3,3)-satom(i,1,1))**2 &
              + 6.0d0 * (satom(i,1,2)**2 + satom(i,2,3)**2 + satom(i,3,1)**2)
         smises = sqrt(smises / 2.0d0)
         write (ifile,'(5f15.8,2f17.8)') qx(i),qy(i),qz(i),enkineach(i)/ev,enpoteach(i)/ev,shydro/mpa,smises/mpa
      enddo

      close (ifile)

      end
